/**************************************************************************
*  Copyright (c) 2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  12.10.2018  mifi  First Version for the i.MX RT1020-EVK board.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include "mcu.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "fsl_semc.h"

/*=======================================================================*/
/*  Prototypes                                                           */
/*=======================================================================*/

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/
#define EXAMPLE_SEMC_CLK_FREQ    CLOCK_GetFreq(kCLOCK_SemcClk)

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

status_t BOARD_InitSEMC(void)
{
   semc_config_t config;
   semc_sdram_config_t sdramconfig;
   uint32_t clockFrq = EXAMPLE_SEMC_CLK_FREQ;
   
   /* Initializes the MAC configure structure to zero. */
   memset(&config, 0, sizeof(semc_config_t));
   memset(&sdramconfig, 0, sizeof(semc_sdram_config_t));

   /* Initialize SEMC. */
   SEMC_GetDefaultConfig(&config);
   SEMC_Init(SEMC, &config);

   /* Configure SDRAM. */
   sdramconfig.csxPinMux = kSEMC_MUXCSX0;
   sdramconfig.address = 0x80000000;
   sdramconfig.memsize_kbytes = 32 * 1024; /* 32MB = 32*1024*1KBytes*/
   sdramconfig.portSize = kSEMC_PortSize16Bit;
   sdramconfig.burstLen = kSEMC_Sdram_BurstLen1;
   sdramconfig.columnAddrBitNum = kSEMC_SdramColunm_9bit;
   sdramconfig.casLatency = kSEMC_LatencyThree;
   sdramconfig.tPrecharge2Act_Ns = 18; /* Trp 18ns */
   sdramconfig.tAct2ReadWrtie_Ns = 18; /* Trcd 18ns */
   sdramconfig.tRefreshRecovery_Ns = (60 + 67);
   sdramconfig.tWriteRecovery_Ns = 12; /* 12ns */
   sdramconfig.tCkeOff_Ns = (1000000000 / clockFrq);
   sdramconfig.tAct2Prechage_Ns = 42;   /* Tras 42ns */
   sdramconfig.tSelfRefRecovery_Ns = 67;
   sdramconfig.tRefresh2Refresh_Ns = 60;
   sdramconfig.tAct2Act_Ns = 60;
   sdramconfig.tPrescalePeriod_Ns = 160 * (1000000000 / clockFrq);
   sdramconfig.refreshPeriod_nsPerRow = 64 * 1000000 / 8192; /* 64ms/8192 */
   sdramconfig.refreshUrgThreshold = sdramconfig.refreshPeriod_nsPerRow;
   sdramconfig.refreshBurstLen = 1;
    
   return SEMC_ConfigureSDRAM(SEMC, kSEMC_SDRAM_CS0, &sdramconfig, clockFrq);
} /* BOARD_InitSEMC */

static void delay (void)
{
    volatile uint32_t i = 0;
    for (i = 0; i < 8000000; ++i)
    {
        __asm("NOP"); /* delay */
    }
}

/*=======================================================================*/
/*  DEFINE: All code exported                                            */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*************************************************************************/
int main (void)
{
   BOARD_InitPins();
   BOARD_BootClockRUN();

   /* Set semc clock to 132Mhz */
   CLOCK_SetMux(kCLOCK_SemcMux, 1);
   CLOCK_SetDiv(kCLOCK_SemcDiv, 2);

   BOARD_InitSEMC();

   __asm("bkpt 0x70");
    
   return(0);
}

/*** EOF ***/
